from __future__ import annotations

import networkit

class KaMinPar:
    """
    The KaMinPar algorithm.
    """

    def __init__(self, graph: networkit.Graph) -> None:
        """
        Initializes the KaMinPar instance with a given graph.

        Args:
            graph (networkit.Graph): The input graph to be partitioned.

        Raises:
            ValueError: If the provided graph is directed.
        """

    def loadGraph(self, graph: networkit.Graph) -> None:
        """
        Loads a new graph into the KaMinPar instance.

        Args:
            graph (networkit.Graph): The graph to be loaded.

        Raises:
            ValueError: If the provided graph is directed.
        """

    def computePartition(self, k: int) -> networkit.Partition:
        """
        Computes a partition of the graph into `k` blocks, allowing an imbalance factor of 3%.

        Args:
            k (int): The number of blocks to partition the graph into.

        Returns:
            networkit.Partition: The computed partition.
        """

    def computePartitionWithEpsilon(self, k: int, eps: float) -> networkit.Partition:
        """
        Computes a partition of the graph into `k` blocks, allowing an imbalance factor of `eps`.

        Args:
            k (int): The number of blocks to partition the graph into.
            eps (float): The allowed imbalance factor.

        Returns:
            networkit.Partition: The computed partition.
        """

    def computePartitionWithFactors(
        self, max_block_weight_factors: list[float]
    ) -> networkit.Partition:
        """
        Computes a partition of the graph into `len(max_block_weight_factors)` blocks,
        using the specified block weight factors.

        Args:
            max_block_weight_factors (list[float]): A list of factors that determine the allowed
                                                    block weights.

        Returns:
            networkit.Partition: The computed partition.
        """

    def computePartitionWithWeights(self, max_block_weights: list[int]) -> networkit.Partition:
        """
        Computes a partition of the graph into `len(max_block_weights)` blocks,
        with specified maximum block weights.

        Args:
            max_block_weights (list[int]): A list of maximum weights for each block.

        Returns:
            networkit.Partition: The computed partition.
        """



# cython: language_level=3
# distutils: language=c++

from libcpp.vector cimport vector

from networkit.graph cimport _Graph, Graph
from networkit.structures cimport _Partition, Partition

cdef extern from "kaminpar-shm/kaminpar.h":
    cdef cppclass _KaMinPar "kaminpar::KaMinPar":
        pass


cdef extern from "kaminpar_networkit.h":
    cdef cppclass _KaMinParNetworKit "kaminpar::KaMinParNetworKit"(_KaMinPar):
        _KaMinParNetworKit(const _Graph &)
        void copyGraph(const _Graph &) except +
        _Partition computePartition(unsigned int) except +
        _Partition computePartitionWithEpsilon(unsigned int, double) except +
        _Partition computePartitionWithFactors(vector[double]) except +
        _Partition computePartitionWithWeights(vector[int]) except +


cdef class KaMinPar:
    cdef _KaMinParNetworKit *thisptr

    def __cinit__(self, Graph G):
        self.thisptr = new _KaMinParNetworKit(G._this)

    def __dealloc__(self):
        del self.thisptr

    def copyGraph(self, Graph G):
        self.thisptr.copyGraph(G._this)

    def computePartition(self, unsigned int k):
        return Partition().setThis(self.thisptr.computePartition(k))

    def computePartitionWithEpsilon(self, unsigned int k, double epsilon):
        return Partition().setThis(self.thisptr.computePartitionWithEpsilon(k, epsilon))

    def computePartitionWithFactors(self, list maxBlockWeightFactors):
        cdef vector[double] v
        for i in range(len(maxBlockWeightFactors)):
            v.push_back(maxBlockWeightFactors[i])
        return Partition().setThis(self.thisptr.computePartitionWithFactors(v))

    def computePartitionWithWeights(self, list maxBlockWeights):
        cdef vector[int] v
        for i in range(len(maxBlockWeights)):
            v.push_back(maxBlockWeights[i])
        return Partition().setThis(self.thisptr.computePartitionWithWeights(v))

